
"""merkury

Usage:
    merkury [-o <file>] <script>

Options:
    -h --help                    Show this screen.
    -o <file>, --output <file>   Specify output file (if missing, prints to stdout)
    --version                    Show version.
"""

from docopt import docopt
from .renderer import produce_report
from .runner import execute

def main():
    """
    Program entrypoint
    """
    args = docopt(__doc__, version="merkury 0.1")
    python_file_path = args.get("<script>")
    output_file_path = args.get("--output")
    code = execute(python_file_path)
    python_file_name = python_file_path.split("/")[-1]
    produce_report(code, python_file_name, output_file_path)

if __name__ == "__main__":
    main()



"""
Reformats code output into report.
"""

from datetime import datetime
from jinja2 import Environment, PackageLoader
from markdown import markdown
import re
import sys

jinja = Environment(
    loader=PackageLoader("merkury", "templates")
)
jinja.filters['markdown'] = lambda content: markdown(content, extensions=['tables', ])

def join_chunks(code):
    """
    Join code nodes without anything printed
    """
    in_chunk = out_chunk = ""
    html = markdown = False
    for input, output in code:
        html = html or any((bool(re.match("^#HTML", line)) for line in input))
        markdown = markdown or any((bool(re.match("^#MARKDOWN", line)) for line in input))
        in_chunk += "".join((line+"\n" for line in input))
        if output != '':
            out_chunk += output
            assert(sum([html, markdown]) <= 1, "Only one option can be specified")
            yield {"in": in_chunk, "out": out_chunk, "html": html, "markdown": markdown}
            in_chunk = out_chunk = ""
            html = markdown = False

def produce_report(code, file_name, output_file_path):
    """
    Main function for transforming raw code
    """
    timestamp = datetime.isoformat(datetime.now())
    chunks = tuple(join_chunks(code))
    data = {"chunks": chunks, "timestamp": timestamp, "file_name": file_name}
    template = jinja.get_template("template.html")
    report = template.render(data)
    if output_file_path:
        with open(output_file_path, 'w') as out:
            out.write(report)
    else:
        sys.stdout.write(report)



"""
Functions for running the script.
"""

import ast
from contextlib import redirect_stdout
from io import StringIO
from itertools import starmap

ENV = {'__name__': '__main__'}

def get_node_line(node):
    """
    Helper for finding starting line for ast node
    Line numbers are 1-indexed, hence conversion
    """
    return (getattr(node, "lineno") - 1)

def get_code_out(node, file_name):
    """
    Get code output from given node
    """
    f = StringIO()
    code = compile(ast.Module([node, ], type_ignores=[]), file_name, 'exec')
    with redirect_stdout(f):
        exec(code, ENV)
    return f.getvalue()

def execute(path):
    """
    Run python script
    """
    file_name = path.split("/")[-1]
    with open(path, 'r') as file:
        source = file.read()
    lines = source.split("\n")
    module = ast.parse(source, file_name)
    start_lines = tuple(map(get_node_line, module.body))
    end_lines = start_lines[1:] + (len(lines), )
    code_inputs = tuple(lines[start:end] for start, end in zip(start_lines, end_lines))
    code_outputs = tuple(starmap(get_code_out, ((node, file_name, ) for node in module.body)))
    return zip(code_inputs, code_outputs)



from setuptools import setup

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(name='merkury',
    version='0.1',
    license='MIT',
    description='Turn Python scripts into HTML reports',
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[],
    author='Piotr Patrzyk',
    url='https://github.com/ppatrzyk/merkury',
    packages=['merkury'],
    package_data={'merkury': ['templates/*', 'templates/assets/*']},
    python_requires='>=3.9',
    install_requires=[
        'docopt>=0.6.2',
        'Jinja2>=3.1.2',
        'Markdown>=3.3.7'
    ],
    entry_points={
        'console_scripts': [
            'merkury=merkury.main:main',
        ],
    },
)
